#pragma once
#include <QVariant>
#include <QJsonObject>
#include <memory>

/// <summary>
/// лʵ
/// </summary>
class Serializable {
public:
	virtual ~Serializable() = default;
	/// <summary>
	/// л
	/// </summary>
	/// <param name="obj"></param>
	virtual void loadFrom(const QJsonObject &obj) = 0;
	/// <summary>
	/// л
	/// </summary>
	/// <param name="obj"></param>
	virtual void saveTo(QJsonObject &obj) const = 0;
	/// <summary>
	/// Ĭϸ
	/// </summary>
	/// <returns></returns>
	virtual std::shared_ptr<Serializable> newDefault() const = 0;
};

/// <summary>
/// 뵥Ԫӿ
/// </summary>
class TranslateUnit : public Serializable {
public:
	virtual ~TranslateUnit() = default;

	/// <summary>
	/// Ԫ
	/// </summary>
	/// <returns></returns>
	virtual QString name() const = 0;

	/// <summary>
	/// һԪֽƫֽ
	/// </summary>
	/// <returns></returns>
	virtual int offsetFromPrevious() const = 0;
	/// <summary>
	/// õֽ
	/// </summary>
	/// <returns></returns>
	virtual int countWithinParse() const = 0;

	/// <summary>
	/// ִн
	/// </summary>
	/// <param name="bytes"></param>
	/// <returns></returns>
	virtual QVariant parse(const QByteArray &bytes) const = 0;
};

class AbstractTranslateUnit : public TranslateUnit {
private:
	QString _name_store;
	int _byte_offset, _byte_count;

public:
	AbstractTranslateUnit(const QString &name);
	/// <summary>
	/// ƫֽ
	/// </summary>
	/// <param name="bytes"></param>
	virtual bool setOffsetFromPrevious(int bytes);
	/// <summary>
	/// ýֽ
	/// </summary>
	/// <param name="bytes"></param>
	virtual bool setCountWithinParse(int bytes);

	// TranslateUnit ===========================
	QString name() const override;
	int offsetFromPrevious() const override;
	int countWithinParse() const override;

	// Serializable ==============================
	void loadFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

#define NAME(u) #u
#define INT32_SAVE(varx) obj[NAME(varx)] = varx;
#define INT32_PEAK(varx) varx = obj[NAME(varx)].toInt();
#define STRLIST_SAVE(list) obj[NAME(list)] = list.join(";");
#define STRLIST_PEAK(list) list = obj[NAME(list)].toString().split(";");

/// <summary>
/// תԴֵδ16ֵĬϷָʽ
/// </summary>
class BytesAsHex : public AbstractTranslateUnit {
public:
	BytesAsHex();

	QVariant parse(const QByteArray& bytes) const override;
	std::shared_ptr<Serializable> newDefault() const override;
};

#include <QHash>
/// <summary>
/// תԴֵΪλ
/// </summary>
class BytesAsBitCombine : public AbstractTranslateUnit {
private:
	QHash<int, QString> _switch_options;

public:
	BytesAsBitCombine();

	bool setSwitchOption(int bit_index, const QString& keyword);
	QHash<int, QString> switchOptions() const;
	virtual void clearOptions();

	// TranslateUnit ============================
	QVariant parse(const QByteArray& bytes) const override;

	// Serializable ==============================
	void loadFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
	std::shared_ptr<Serializable> newDefault() const override;
};

/// <summary>
/// תԴΪ
/// </summary>
class BytesAsInteger : public AbstractTranslateUnit {
private:
	bool unsigned_mark = false;

public:
	BytesAsInteger();

	void setUnsignedMark(bool ste);
	bool unsignedMark() const;

	// TranslateUnit ============================
	bool setCountWithinParse(int bytes) override;
	QVariant parse(const QByteArray& bytes) const override;

	// Serializable ==============================
	void loadFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
	std::shared_ptr<Serializable> newDefault() const override;
};

#define STRING_PEAK(codec_name) codec_name = obj[NAME(codec_name)].toString();
#define STRING_SAVE(codec_name) obj[NAME(codec_name)] = codec_name;
/// <summary>
/// תԴΪַ
/// </summary>
class BytesAsString : public AbstractTranslateUnit {
private:
	QTextCodec *_conv_with = nullptr;

public:
	BytesAsString();

	void setStrCodec(QTextCodec *ins);
	QString codecName() const;

	// TranslateUnit ============================
	QVariant parse(const QByteArray& bytes) const override;

	// Serializable ==============================
	void loadFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
	std::shared_ptr<Serializable> newDefault() const override;
};